// Sortierungsgeschwindigkeit testen

#include <stdio.h>

#include "../hash/hash.hh"

#include <unistd.h>
#include <time.h>


// kopiert die Daten !!!


// Heapsort Zeitbedarf:  f(n) = ( x lg n )

#define ARRAYTYPE < unsigned int >

hsarray ARRAYTYPE *sarray;


// Ausgaben
int verbose;


// ### fr hsort ###

// Array ausgeben
void
ausgabearray (unsigned int anz)
{
  unsigned int data;

  cout << " " << sarray->getfirst ();

  for (unsigned int x = 1; x < anz; x++)
    {
      if (sarray->getnext (data))
	cout << " " << data;
      else
	cout << "Error getnext=0!\n";
    }

  cout << endl;
}



// #########  Zufallszahlengenerator  ############
void
initarray (unsigned int anz, unsigned int *a)
{
  unsigned int x;
  for (x = 0; x < anz; x++)
    a[x] = rand () + 1;		// keine Nullen !!!
}


void
ausgabe (unsigned int anz, unsigned int *a)
{
  unsigned int x;

  for (x = 0; x < anz; x++)
    {
      cout << " " << a[x];
    }

  cout << endl;
}


// ##################### fr qsort ########################
unsigned int
qpart (unsigned int *A, unsigned int v, unsigned int b)
{
  unsigned int x;
  int i, j;

  x = A[v];
  i = v - 1;
  j = b + 1;

  while (1)
    {
      do
	j--;
      while (!(A[j] <= x));

      do
	i++;
      while (!(A[i] >= x));

      // Sortorder kann nicht gendert werden da sonst die Suchfunktion nicht geht
      if (i < j)
	{
	  // ist Gleichschnell aber schner!
	  A[i] ^= A[j];
	  A[j] ^= A[i];
	  A[i] ^= A[j];
	}
      else
	{
	  return j;
	}
    }
}

void
shfqsort (unsigned int *A, unsigned int v, unsigned int b)
{
  unsigned int q;

  if (v < b)
    {
      q = qpart (A, v, b);
      shfqsort (A, v, q);
      shfqsort (A, q + 1, b);
    }
}

void
hfqsort (unsigned int *A, unsigned int l)
{
  shfqsort (A, 0, l - 1);	// Array Grenzen 
}

// #######################################################################

void
execqsort (unsigned int anz, unsigned int *a)
{
  // Standart Implementation
  hfqsort (a, anz);
}


int
testesort (unsigned int anz, unsigned int *a)
{
  bool err = false;
  unsigned int x, al;

  al = sarray->getfirst ();

  for (x = 0; x < anz; x++)
    {
      if (a[x] != al)
	{
	  err = 255;
	  fprintf (stderr, "Fehler bei: %3d q=0x%08x a=0x%08x\n", x, a[x],
		   al);
	}
      sarray->getnext (al);
    }

  cout << endl;

  return err;
}

// ############ Mehrfachtests ############################

clock_t
exhsort (int l, unsigned int *a)
{
  int x;
  clock_t ha, he, hd;

  hd = 0;

  for (x = 0; x < l; x++)
    {
      sarray->copy (a);

      ha = clock ();
      sarray->heapsort ();
      he = clock ();

      hd += he - ha;
    }

  return hd;
}

clock_t
exqsort (int l, unsigned int anz, unsigned int *a, unsigned int *b)
{
  int x;
  clock_t ha, he, hd;

  hd = 0;

  for (x = 0; x < l; x++)
    {
      memcpy (a, b, anz * sizeof (unsigned int));

      ha = clock ();
      execqsort (anz, a);
      he = clock ();

      hd += he - ha;
    }

  return hd;
}


// teste sort
int
main (int argc, char *argv[])
{
  unsigned int *a, *b;
  int e, x, loops;

  clock_t hq, hd;

  unsigned long anz, anzeige;	// Anzahl der Zahlen

  anz = 0xff;
  verbose = false;
  e = 0;
  loops = anzeige = 10;

  // lese das Argument #################
  if (argc < 2)
    {
    hilfe:
      cout << "Bitte geben Sie die Anzahl der Values an!\n";
      cout << "hsort [-vN|VN ] [-h|?] [-a N ] [-l N] \n";
      cout << "v = Verbose Anzeigeanzahl N\n";
      cout << "a = Anzahl N\n";
      cout << "l = Schleifen, wieoft wird sortiert\n";
      return -1;
    }

  for (e = 1; e != 0;)
    {
      switch ((x = getopt (argc, argv, "v:V:a:b:l:h?")))
	{
	case 'v':
	  verbose = true;
	  anzeige = atol (optarg);
	  break;
	case 'V':
	  verbose = 2;
	  anzeige = atol (optarg);
	  break;
	case 'h':
	case '?':
	  goto hilfe;
	  break;
	case 'a':
	  anz = atol (optarg);
	  break;
	case 'l':
	  loops = atol (optarg);
	  break;

	case -1:
	  e = 0;
	  break;
	}
    }

  e = 0;
  a = new unsigned int[anz];
  b = new unsigned int[anz];

  sarray = new hsarray ARRAYTYPE (anz);

  if (a)
    {
      if (verbose)
	cout << "Init Array, Anz=" << anz << endl;

      initarray (anz, a);
      memcpy (b, a, anz * sizeof (unsigned int));

      if (verbose)
	{
	  cout << "Ausgabe der ersten Zahlen: \n";
	  ausgabe (anzeige, a);
	  cout << "Sortiere Hsort...\n";
	}

      hd = exhsort (loops, a);

      if (verbose)
	{
	  cout << "Diff: " << hd << endl;
	  cout << "Ausgabe der ersten sortierten Zahlen: \n";
	  ausgabearray (anzeige);
	  cout << "Sortiere Qsort...\n";
	}

      hq = exqsort (loops, anz, a, b);

      if (verbose)
	{
	  cout << "Diff: " << hq << endl;
	  cout << "Ausgabe der ersten sortierten Zahlen: \n";
	  ausgabe (anzeige, a);
	}


      e = testesort (anzeige, a);
      cout << "Ergebniss:\n";
      cout << "Errors: " << e << " !\n";
      cout << "Zeit qsort: " << hq << endl;
      cout << "Zeit hsort: " << hd << endl;

    }
  else
    {
      cerr << "Konnte nicht gengend " << anz << " Integer mallocen!\n";
    }

  return e;
}
