/* 
 * Test und Dokumentationsprogramm zur hash.h
 * 
 * Autor:     Helmut Fahrion 
 * Email:     hefa@gmx.net
 * GPG-KeyID: 12622C98
 * Copyright: Mach was Du willst damit!
 *
 */


#include <unistd.h>
#include "hash.hh"

// je nach Speicheraustattung kann man die Tests auch bertreiben
#define  ANZ  0x1ffff

// die Dimension sollte auf den Schlssel angepasst werden, damit
// ist die Speicherauslastung optimal
#define DIMS   5
#define FORMAT "%05x"


// Testschleifen um Memory Lcher zu finden
#define SCHLEIFEN 100

// Nett zu anderen Prozessen
#define ZSLEEP 200
#define Z 3

int erroranz;

typedef unsigned long unsl;

int
main (int argc, char *argv[])
{
  int a;
  int *p, x, y, schleifen;
  long i;
  errorenum e;
  larray < int *>*liste;
  laperror err;


  char key[0xf];

  //      data type , Dimensionen (keylen)
  //xdhashstr < int *, 8 > hh;
  xdhstrh < int *, DIMS > hh;

  erroranz = 0;

  // lese den Parameter ein.
  if (argc < 2)
    {
      cout << "bitte geben Sie die Anzahl der Schleifen mit an!\n";
      return -1;
    }

  schleifen = 0;
  schleifen = atoi (argv[1]);

  if (schleifen <= 0)
    return -1;

  erroranz = 0;

  // gehe Schleifen durch!
  for (i = 0; i < schleifen; i++)
    {

      cout << "\ninsert: " << ANZ << " items: ";

      //usleep (Z);
      // Zahlen einfgen
      for (a = 0; a < ANZ; a++)
	{
	  p = new int (-a - 1);

	  // Wichtig ist hier das die Zeichen nur von 0...9...a...z gehen
	  // drfen, also von dez. 48 bis dez. 122
	  sprintf (key, FORMAT, a);

	  // Erst ein sprintf in einen String und dann die Hash fr Strings 
	  // zu verwenden ist nur fr dieses Demo sinnvoll. Eigentlich kann
	  // man hier gleich die xdhashint verwenden. 
	  if ((e = hh.insert (key, p)))	// key , data
	    {
	      erroranz++;
	      err.str (e, "es wird nicht eingefgt");
	    }
	  else
	    printf (FORMAT " ", *p);

	}
      cout << "\n*** Fehleranzahl ist:" << erroranz << "!, " << "Schleife " <<
	i << " Super! ***\n";
      if (erroranz)
	exit (-1);

      usleep (ZSLEEP);


      // Zahlen wieder suchen
      cout << "\nfinde: ";
      for (a = 0; a < ANZ; a++)
	{
	  //usleep (Z);
	  sprintf (key, FORMAT, a);
	  hh.search (&p, key, &e);

	  if (p)
	    {
	      cout << *p << "  ";

	      // geht nicht bei null
	      if (-a - 1 != *p)
		erroranz++;

	      if (erroranz)
		{
		  cout << " Error Key=" << (int) a << " Data=" << *p << endl;
		  exit (-1);
		}

	    }
	  else
	    erroranz++;
	}

      cout << "\n*** Fehleranzahl ist:" << erroranz << "!, " << "Schleife " <<
	i << " Super! ***\n";
      if (erroranz)
	exit (-1);

      usleep (ZSLEEP);
      cout << endl << "iteriere: ";


      // die Funktion getall gibt eine Liste zurck,
      // sollten Zeiger auf die Daten sein.
      liste = new larray < int *>(ANZ + 3);

      hh.getall (liste, &e);

      if (liste)
	{
	  x = *((int *) liste->getfirst ());
	  cout << "f: " << x << "\n";

	  // berprfe gleich mit der Suchfunktion
	  while (liste->isnext ())
	    {
	      //y = *((int *) liste->getnext ());
	      liste->getnext (p);
	      y = *p;
	      cout << "l: " << y << " ";
	      //usleep (Z);
	    }
	}
      else
	{
	  erroranz++;
	}

      cout << "\n*** Fehleranzahl ist:" << erroranz << "!, " << "Schleife " <<
	i << " Super! ***\n";
      if (erroranz)
	exit (-1);

      // Speicher wieder freigeben
      delete liste;

      cout << endl;

      usleep (ZSLEEP);


      cout << endl << "delete: ";
      e = NOOK;

      // einzelnen Elemente wieder freigeben,
      // auch Achtung beim delete auf den Hash, dieser
      // Hash gibt die Daten frei!, siehe hash.h !

      /*
         for (a = 0; a < ANZ; a++)
         {
         //usleep (Z);
         sprintf (key, "%08x", a);
         hh.destroy (key, &e);

         cout << (int) a << " ";
         if (e != ISOK)
         {
         erroranz++;
         }
         }
       */

      hh.delall ();

      liste = new larray < int *>(ANZ + 3);


      cout << "\n** hh.getall! ***\n";
      hh.getall (liste, &e);

      if (liste)
	{
	  if ((p = (int *) liste->getfirst ()))
	    {
	      cout << "f: " << *p << "\n";
	      erroranz++;

	      // berprfe gleich mit der Suchfunktion
	      while (liste->isnext ())
		{
		  //y = *((int *) liste->getnext ());
		  liste->getnext (p);
		  y = *p;

		  cout << "l: " << y << " ";
		  //usleep (Z);
		  erroranz++;
		}
	    }
	}

      cout << "\n*** Fehleranzahl deleteall() ist:" << erroranz << "!, " <<
	"Schleife " << i << " Super! ***\n";
      if (erroranz)
	exit (-1);

      // Speicher wieder freigeben
      delete liste;

      cout << "\n*** Ende: Fehleranzahl ist:" << erroranz << "!, " <<
	"Schleife " << i << " Super! Schleifenende!***\n";
      usleep (ZSLEEP);

      if (erroranz)
	{
	  exit (-1);
	}

    }				// schleife
  return 0;
}
